#ifndef READ_H
#define READ_H

#include <istream>
#include "logging.h"

namespace io {

	template<typename T>
	void read_to(std::istream& in, T* buffer, size_t n = 1)
	{
		in.read(reinterpret_cast<char*>(buffer), n * sizeof(T));
		CHECK(!in.fail()) << "failed to read input";
	}

	template<typename T, typename std::enable_if<std::is_fundamental<T>::value, T>::type* = nullptr>
	T read(std::istream& in)
	{
		T* buffer = new T;
		read_to(in, buffer);
		return *buffer;
	}

	template<typename T>
	std::unique_ptr<T[]> read_array(std::istream& in, size_t n)
	{
		auto array = std::make_unique<T[]>(n);

		/*int i = n;
		for (int i = 0; i < n; i++)
		{
			read_to(in, &array[i]);
		}*/
		read_to(in, array.get(), n);

		return std::move(array);
	}

}
#endif // !READ_H
